from datetime import timedelta
import time
import datetime

class communication_link:
    """
    Represents communication link defined with unique id,
    capacity and current traffic consumption
    """
    def __init__(self, identifier, capacity):
        self.identifier = identifier
        self.capacity = capacity
        self.consumption = 0

class DecisionModule:
    """
    Decision module is responsible to determine if 
    there is enough bandwidth available at links
    provided.
    """
    def __init__(self, links):
        self.links = links
        
    def FlowCanBeAddedAsPrivileged(self, app_type, requested_bw, links):
        for link in links:
            if (self.links[link].consumption + requested_bw > 
                self.links[link].capacity):
                return False

        for link in links:
            self.links[link].consumption = self.links[link].consumption + requested_bw

        return True

# Decision module tests

def TestCase1():
    print "Test Case 1: Successfull adding of single flow"
    links = { 1:communication_link(1, 10),
              2:communication_link(2, 7),
              3:communication_link(3, 8),
              4:communication_link(4, 12),
              5:communication_link(5, 14) }

    decision_module = DecisionModule(links)

    assert decision_module.AddPrivilegedFlow(1, 2, [1, 2, 3]) == True, "Test Case 1 failed."

def TestCase2():
    print "Test Case 2: Failed adding of second single flow"
    links = { 1:communication_link(1, 10),
              2:communication_link(2, 7),
              3:communication_link(3, 8),
              4:communication_link(4, 12),
              5:communication_link(5, 14) }

    decision_module = DecisionModule(links)

    assert decision_module.AddPrivilegedFlow(1, 2, [1, 2, 3]) == True, "Test Case 2 failed."

    assert decision_module.AddPrivilegedFlow(1, 7, [1, 2, 3]) == False, "Test Case 2 failed."

def TestCase3():
    print "Test Case 3: Masure algorithm throughput"
    links_number = 1000000

    links = { }

    # prepare links to 
    for link_id in range (1, links_number):
        links[link_id] = communication_link(link_id, 10)

    decision_module = DecisionModule(links)

    a = datetime.datetime.now()

    for x in range (1, 10):
        decision_module.AddPrivilegedFlow(1, 0, range (1, links_number))

    b = datetime.datetime.now()

    c = b - a

    print c.total_seconds()

def main():
    TestCase1()
    TestCase2()
    TestCase3()

if __name__ == '__main__':
    main()

