from threading import Thread
from time import sleep
import socket
import sys
import pickle
from decision_module import *
from request import SGRequest
import struct

def get_comm_links():
  """
  Wrapper function defining communication link bandwidth
  for specific test case.
  """
  comm_links = { 1:communication_link(1, 10),
                 2:communication_link(2, 10),
                 3:communication_link(3, 10) }

  return comm_links

class TCPServer:
  """
  Hosts endpoint and received requests from SG applications.
  
  Based on decision_module response, it instructs controller
  to update the switches.
  """
  _singletone = None
  started = False
  sock = None            # socket that will be used to communicate with SG application

  application_types = { 1:("10.0.0.1", 5059),
                        2:("10.0.0.1", 5061),
                        3:("10.0.0.2", 5060) }

  def __init__(self):
    self.switches = []
    self.applications = []
    self.privileged_app_types = set()
                        
  def _create_socket(self, port):
    self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    print 'Socket created'

    try:
      self.sock.bind(('', port))
    except socket.error as msg:
      print 'Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1]
      sys.exit()

    print 'Socket bind complete'
    
  def enforce_rule_update(self, app_types):
    self.privileged_app_types = set()
    for app_type in app_types:
      ip = TCPServer.application_types[app_type.type_id][0]
      port = TCPServer.application_types[app_type.type_id][1]
      self.privileged_app_types.add(ip + ':' + str(port))
      for switch in self.switches:
        print "Deleting flows: " + ip + ":" + str(port) + "."
        # Are rules deleted and added on the first pachage
        switch.delete_flow_rules(ip, port)
        
        # Are rules installed proactivly, if yes 
        switch.update_flow_rules(ip, port)
    
  def is_traffic_privileged(self, ip_address, port):
   print "Checking if traffic is privileged: " + str(ip_address) + ":" + str(port)
   if (str(ip_address) + ':' + str(port)) in self.privileged_app_types:
     print "Flow threated as privileged."
     return True 
   return False

  def _read_from_socket(self, sock, length):
    data = ''
    while len(data) < length:
      data += sock.recv(length - len(data))
    return data

  def _read_request_from_socket(self, sock):
        
        # read type_id
        data = self._read_from_socket(sock, 4)
        type_id = struct.unpack('!i', data[:4])[0]
        
        # read priority
        data = self._read_from_socket(sock, 4)
        priority = struct.unpack('!i', data[:4])[0]
        
        # read bandwidth
        data = self._read_from_socket(sock, 4)
        bandwidth = struct.unpack('!f', data[:4])[0]
 
        data = self._read_from_socket(sock, 4)
        links_num = struct.unpack('!i', data[:4])[0]

        links = []
        for x in range(links_num):
          data = self._read_from_socket(sock, 4)
          link_id = struct.unpack('!i', data[:4])[0]
          links.append(link_id)

        request = SGRequest(type_id, priority, bandwidth, links)
        return request

  def _client_handler(self, sock):
    try:
      while 1:
        # receive request
        request = self._read_request_from_socket(sock)

        print "Received request: type " + str(request.type_id) + ", priority: " + str(request.priority) + ", bandwidth: " + str(request.bandwidth) + ", links: " + str(request.links)

        # process request
        privileged_apps, unprivileged_apps = self.AddModifyFlow(request)
 
    except Exception as e:
      print(e)
      print "Client disconnected."

  def AddModifyFlow(self, request):
      privileged_apps, unprivileged_apps = self.AddModifyFlow_DM(request)
      self.enforce_rule_update(privileged_apps)
      return privileged_apps, unprivileged_apps

  def AddModifyFlow_DM(self, request):
        """
        Process an request based on the Algorithm 2, entrace
        to decision module.
        """
  
        #print "Process request: " + str(request)
  
        decision_module = DecisionModule(get_comm_links())
        
        for i, a in enumerate(self.applications):
          if a.type_id == request.type_id:
            del self.applications[i]
        
        self.applications.append(request)
        self.applications.sort(key=sort_key)
        
        privileged_apps = []
        unprivileged_apps = []
        
        for app in self.applications:
          if decision_module.FlowCanBeAddedAsPrivileged(request.type_id, request.bandwidth, request.links):
            privileged_apps.append(app)
            #print "Adding app " + str(app.type_id) + " to privileged list."
          else:
            unprivileged_apps.append(app)
            #print "Adding app " + str(app.type_id) + " to unprivileged list."
            
        return privileged_apps, unprivileged_apps
      
  def _start_server(self, port):
    listen_port = port
    listen_timeout = 10
    self._create_socket(listen_port)

    #Start listening on socket
    self.sock.listen(listen_timeout)
    
    while 1:
    #wait to accept a connection - blocking call
      print 'Waiting for SG applications to connect.'
      conn, addr = self.sock.accept()
      print 'Connected SG application from: ' + addr[0] + ':' + str(addr[1])

      thread = Thread(target = self._client_handler, args = (conn, ))
      thread.start()

  def start_server(self, port):
    if TCPServer.started == True:
        return
    TCPServer.started = True 
    thread = Thread(target = self._start_server, args = (port, ))
    thread.start()

  def add_switch(self, switch):
    self.switches.append(switch)
    
  @classmethod
  def create_instance():
    if _singletone == None:
      _singletone = TCPServer(decision_module)
    return singletone

  @classmethod
  def instance(cls):
    if cls._singletone == None:
        cls._singletone = TCPServer()
    return cls._singletone

def sort_key(element):
  return element.priority
               
class DynamicPrioritizationController:
    def __init__(self):
        print "DynamicPrioritizationController"

def TestCase1():
    """
    Test if the least important application will be treated
    as unprivileged in case of insufficient bandwidth.
    """
    print "TestCase1"
    server = TCPServer();

    request_1 = SGRequest(1, 1, 5, [1, 2, 3])
    request_2 = SGRequest(2, 2, 5, [1, 2, 3])
    request_3 = SGRequest(3, 3, 5, [1, 2, 3])
    
    pa, ua = server.AddModifyFlow(request_1)

    pa, ua = server.AddModifyFlow(request_2)
    
    pa, ua = server.AddModifyFlow(request_3)
    
    assert server.is_traffic_privileged(TCPServer.application_types[1][0], TCPServer.application_types[1][1]), "Application should be privileged."

    assert server.is_traffic_privileged(TCPServer.application_types[2][0], TCPServer.application_types[2][1]), "Application should be privileged."

    assert server.is_traffic_privileged(TCPServer.application_types[3][0], TCPServer.application_types[3][1]) == False, "Application should not be privileged."
    
def TestCase2():
    """
    Test if the least important application will be treated
    as unprivileged in case of insufficient bandwidth - when
    apps are added out of order.
    """
    print "TestCase2"
    server = TCPServer();

    request_1 = SGRequest(1, 1, 5, [1, 2, 3])
    request_2 = SGRequest(2, 2, 5, [1, 2, 3])
    request_3 = SGRequest(3, 3, 5, [1, 2, 3])
    
    pa, ua = server.AddModifyFlow(request_1)

    pa, ua = server.AddModifyFlow(request_3)
    
    assert server.is_traffic_privileged(TCPServer.application_types[1][0], TCPServer.application_types[1][1]), "Application should be privileged."

    assert server.is_traffic_privileged(TCPServer.application_types[3][0], TCPServer.application_types[3][1]), "Application should be privileged."
    
    pa, ua = server.AddModifyFlow(request_2)

    assert server.is_traffic_privileged(TCPServer.application_types[2][0], TCPServer.application_types[2][1]), "Application should be privileged."

    assert server.is_traffic_privileged(TCPServer.application_types[3][0], TCPServer.application_types[3][1]) == False, "Application should be unprivileged."

    
    print "Privileged applications: " + str(pa)
    print "Unprivileged applications: " + str(ua)
      
def main():
    TestCase1()
    TestCase2()
  
if __name__ == '__main__':
    main()

