from mininet.log import info, error, debug, output, warn

class CommandGenerator:
    """
    Contains command string templates at single place.
    """

    _STREAM_TRAFFIC_GENERATOR_COMMAND_TEMPLATE = "{0}/stream_traffic_generator {1} {2} {3} >> {4}/stream_traffic_generator.{5}.{1} &"
    _STREAM_TRAFFIC_SINK_COMMAND_TEMPLATE = "{0}/stream_traffic_sink {1} {2} >> {3}/stream_traffic_sink.{5}.{4} &"

    _DEVICE_SIMULATOR_COMMAND_TEMPLATE = '{0}/device_simulator {1} >> {2}/device_simulator.{3}.{1} &'
    _COMMAND_SIMULATOR_COMMAND_TEMPLATE = '{0}/command_simulator {1} {2} {3} {4} {5} {6} >> {7}/command_simulator.{6} &'
 
#    _QUEUE_COMMAND_TEMPLATE = "ovs-vsctl -- set port {0} qos=@{1} -- --id=@{1} create QoS type=linux-htb other-config:max-rate={3} queues=0=@{2} -- --id=@{2} create Queue other-config:min-rate=0 other-config:max-rate={3}"

    _QUEUE_COMMAND_TEMPLATE = "ovs-vsctl -- set port {0} qos=@{1} -- --id=@{1} create QoS type=linux-htb other-config:max-rate={3} queues=0=@{2},1=@q1 -- --id=@{2} create Queue other-config:min-rate=0 other-config:max-rate={3} -- --id=@q1 create Queue other-config:min-rate={3} other-config:max-rate={3}"


    _POLICING_INGRESS_COMMAND_TEMPLATE = "ovs-vsctl set interface {0} ingress_policing_rate={1}"
    _POLICING_BURST_COMMAND_TEMLPATE = "ovs-vsctl set interface {0} ingress_policing_burst={1}"

    _CLEAR_PROVIDER_TEMPLATE = 'sudo ovs-vsctl clear Bridge {0} netflow'
    _CREATE_PROVIDER_TEMPLATE = 'sudo ovs-vsctl -- set Bridge {0} netflow=@nf0 -- --id=@nf0 create NetFlow targets=\\"127.0.0.1:2055\\" add_id_to_interface=false active_timeout=1'
   
    @classmethod
    def stream_traffic_generator(cls, simulator_folder, output_folder, port, bandwidth, source_name, load_type):
        command = cls._STREAM_TRAFFIC_GENERATOR_COMMAND_TEMPLATE.format(simulator_folder, port, bandwidth, '~/sm_simulator/' + load_type, output_folder, source_name)
        info('stream_traffic_generator: ', command, '\n')
        return command
        
    @classmethod
    def stream_traffic_sink(cls, simulator_folder, output_folder, ip, port, destination_name):
        command = cls._STREAM_TRAFFIC_SINK_COMMAND_TEMPLATE.format(simulator_folder, ip, port, output_folder, destination_name, port)
        info('stream_traffic_sink: ', command, '\n')
        return command

    @classmethod
    def device_simulator(cls, simulator_path, port, destination_name, output_path):
        command = cls._DEVICE_SIMULATOR_COMMAND_TEMPLATE.format(simulator_path, port, output_path, destination_name)
        info('device_simulator: ', command, '\n')
        return command
        
    @classmethod
    def command_simulator(cls, simulator_path, output_path, execution_time, commanding_period, command_size, commands_number, ip_address, port, destination_name):

        command = cls._COMMAND_SIMULATOR_COMMAND_TEMPLATE.format(simulator_path,
                                                                 execution_time,
                                                                 commanding_period,
                                                                 command_size,
                                                                 commands_number,
                                                                 ip_address,
                                                                 port,
                                                                 output_path)
        info('dr_simulator_provider: ', command, '\n')
        return command       

    @classmethod
    def queue_command(cls, interfaceName, maxRateBytes):
        command = cls._QUEUE_COMMAND_TEMPLATE.format(interfaceName, 'qos_example', 'q0', maxRateBytes)
        info("queue_command", command, '\n')
        return command

    @classmethod
    def policing_ingress_rate_command(cls, interfaceName, rateKb):
        command = cls._POLICING_INGRESS_COMMAND_TEMPLATE.format(interfaceName, rateKb)
        info("policing_ingress_rate_command", command, '\n')
        return command

    @classmethod
    def policing_ingress_burst_command(cls, interfaceName, burstKb):
        command = cls._POLICING_BURST_COMMAND_TEMLPATE.format(interfaceName, burstKb)
        info("policing_ingress_burst_command", command, '\n')
        return command
    
    @classmethod    
    def create_provider_command(cls, bridge_name):
        command = cls._CREATE_PROVIDER_TEMPLATE.format(bridge_name)
        info("create_provider_command", command, '\n')
        return command

    @classmethod    
    def clear_provider_command(cls, bridge_name):
        command = cls._CLEAR_PROVIDER_TEMPLATE.format(bridge_name)
        info("clear_provider_command", command, '\n')
        return command


