#include <stdio.h> //printf
#include <string.h>    //strlen
#include <sys/socket.h>    //socket
#include <sys/time.h>
#include <arpa/inet.h> //inet_addr
#include <stdlib.h>
#include <time.h>
#include "common.h"

void usage();

void DumpRequestSent(struct tm tmRequestStart);

void DumpStats(struct tm tmRequestStart, struct tm tmRequestEnd, int requestId, double sendingTime, double receiveTime, int executionTime);

int main(int argc , char *argv[])
{
    if (argc != 7)
    {
        debug("Invalid number of arguments.\n");
        usage();
        return 0;
    }

    // Parse execution time and convert seconds to us
    int execution_time = atoi(argv[1]) * 1000 * 1000;
    int period = atoi(argv[2]) * 1000 * 1000;
    int command_size = atoi(argv[3]);
    int commands_number = atoi(argv[4]);
    char remote_ip_address[IP_ADDRESS_SIZE];
    strcpy(remote_ip_address, argv[5]);
    int remote_port = atoi(argv[6]);

    debug_n("DR Simulator started with following parameters:\n");
    debug_n("execution time: %d, period: %d, command size: %d, commands number: %d, remote ip address: %s, remote port: %d.\n", execution_time, period, command_size, commands_number, remote_ip_address, remote_port);

    int sock = persistent_connect(remote_ip_address, remote_port);
    if (sock < 0)
    {
        debug_n("Error during connect. Exiting.");
        return 1;
    }

    start_tcp_stats_collecting(sock);

    int request_id = 0;
    char *command = (char*) malloc(command_size);
    int response_size = 512;
    char *response = (char*) malloc(response_size);

    debug_n("Requst sent, Response received, Request ID, Communication time, Sending time, Receive time");
    for(int i = 0; i < commands_number; i++)
    {
        int * i_cmd = (int*) command;
        i_cmd[0] = request_id;
        i_cmd[1] = execution_time;
        i_cmd[2] = response_size;

        time_t t_request_start = time(NULL);
        struct tm tm_request_start = *localtime(&t_request_start);

        struct timespec ts_1, ts_2;
        clock_gettime(CLOCK_MONOTONIC, &ts_1);

        if (SendMessage(sock, command, command_size))
        {
            debug_n("Error sending message.");
            return 1;
        }

        clock_gettime(CLOCK_MONOTONIC, &ts_2);

        double sending_time = CalculateElapsedTime(ts_1, ts_2);

        clock_gettime(CLOCK_MONOTONIC, &ts_1);

        int received_response_size = ReceiveMessage(sock, response, response_size);

        if (received_response_size < 0)
        {
            debug_n("Received too big message.");
            return 2;
        }

        if(received_response_size != response_size)
        {
            debug_n("Response size different than the one sent: %d vs %d.", received_response_size, response_size);
        }

        clock_gettime(CLOCK_MONOTONIC, &ts_2);
        double receive_time = CalculateElapsedTime(ts_1, ts_2);

        time_t t_request_end = time(NULL);
        struct tm tm_request_end = *localtime(&t_request_end);

        DumpStats(tm_request_start, tm_request_end, request_id, sending_time, receive_time, execution_time);

        int sleep_time = period - receive_time - sending_time; 
        if (sleep_time > 0)
        {
            usleep(sleep_time);
        }

        request_id++;

        if (request_id % 1000 == 0)
        {
            debug_n("Number of responses received is %d.", request_id);
        }
    }

    return 0;
}

void usage()
{
	debug_n("Application for sending commands to command simulator.");
	debug_n("Usage:\n");
	debug_n("command_simulator execution_time commanding_period command_size commands_number remote_ip_address remote_port");
}

void DumpRequestSent(struct tm tmRequestStart)
{

	debug("%d-%d-%d %d:%d:%d: Request sent.\n", tmRequestStart.tm_year + 1900, tmRequestStart.tm_mon + 1, tmRequestStart.tm_mday, tmRequestStart.tm_hour, tmRequestStart.tm_min, tmRequestStart.tm_sec);
}

void DumpStats(struct tm tmRequestStart, struct tm tmRequestEnd, int requestId, double sendingTime, double receiveTime, int executionTime)
{
	debug_n("%d-%d-%d %d:%d:%d, %d-%d-%d %d:%d:%d, %d, %lf, %lf, %lf", 
		tmRequestStart.tm_year + 1900, tmRequestStart.tm_mon + 1, tmRequestStart.tm_mday, tmRequestStart.tm_hour, tmRequestStart.tm_min, tmRequestStart.tm_sec,
		tmRequestEnd.tm_year + 1900, tmRequestEnd.tm_mon + 1, tmRequestEnd.tm_mday, tmRequestEnd.tm_hour, tmRequestEnd.tm_min, tmRequestEnd.tm_sec,
		requestId, sendingTime + receiveTime - executionTime, sendingTime, receiveTime - executionTime);
}


