#include "common.h"

// returns timespan in us between provided start and end timestamps
double CalculateElapsedTime(struct timespec start, struct timespec end)
{
	double nsec = end.tv_nsec - start.tv_nsec;
        double elapsedTime = 0;
        if(nsec < 0)
        {
		nsec = 1000000000 + end.tv_nsec - start.tv_nsec;
                return elapsedTime = (end.tv_sec - start.tv_sec - 1) * 1000 * 1000 + (double) nsec / 1000;
        }
	elapsedTime = (end.tv_sec - start.tv_sec) * 1000 * 1000 +  (double) nsec / 1000;
	return elapsedTime;
}

// Returns when provided buffer of len sizeToSen
// is sent to socket or when error has happened.
// Returns zero when there was no error or !=0
// otherwise.
int Send(int sock, void* buffer, int sizeToSend)
{
	int requestDataSent = 0;
	char errorSending = 0;
	while(requestDataSent != sizeToSend)
	{
		int data_sent = send(sock, (char *) buffer + requestDataSent , sizeToSend - requestDataSent , 0);

		if(data_sent < 0)
		{
		        debug("Send failed. Error: %s\n", strerror(errno));
			errorSending = 1;
		        break;
		}

		requestDataSent += data_sent;
	}
	return errorSending;
}


// Returns when provided sizeToRead bytes are
// received from socket or when error has happened.
// Returns zero when there was no error or !=0
// otherwise.
int Receive(int sock, void* buffer, int sizeToRead)
{
	int errorReceiving = 0;
	int responseDataReceived = 0;
        while(responseDataReceived != sizeToRead)
        {
                //Receive a reply from the server
                int data_received = recv(sock, (char *) buffer + responseDataReceived, sizeToRead - responseDataReceived , 0);

                if(data_received < 0)
                {
                        debug("recv failed. Error: %s\n", strerror(errno));
			errorReceiving = 1;
                        break;
                }
                else if (data_received == 0)
                {
                        debug("recv returned zero for socket %d, client disconnected. Error: %s\n", sock, strerror(errno));
			errorReceiving = 1;
                        break;
                }

                responseDataReceived += data_received;
        }
	return errorReceiving;
}

int persistent_bind(int socket, int port)
{
    struct sockaddr_in server;

    //Prepare the sockaddr_in structure
    server.sin_family = AF_INET;
    server.sin_addr.s_addr = INADDR_ANY;
    server.sin_port = htons( port );

    while (1)
    {
        int returnValue = bind(socket,(struct sockaddr *)&server , sizeof(server));
        // Loop if address already in use error occures. This usually happens when
        // the application is started again on the same port in short time-span.
        if (returnValue < 0 && errno ==  EADDRINUSE )
        {
            usleep(100*1000);
            continue;
        }
        else if( returnValue < 0 )
        {
            //print the error message
            debug("bind failed. Error: %s, error code %d\n", strerror(errno), errno);
            return 1;
        }
        else
        {
            break;
        }
    }
}

int persistent_connect(char* remote_ip_address, int remote_port)
{
    // Create socke
    int sock = socket(AF_INET , SOCK_STREAM , 0);
    if (sock == -1)
    {
        return sock;
    }

    struct sockaddr_in server;
    server.sin_addr.s_addr = inet_addr(remote_ip_address);
    server.sin_family = AF_INET;
    server.sin_port = htons(remote_port);

    //Connect to remote server
    debug_n("Connecting to socket %d.", sock);
    while (connect(sock, (struct sockaddr *)&server, sizeof(server)) < 0)
    {
        debug_n("connect failed with error %s. Retrying...", strerror(errno));
        usleep(1000*1000);
    }
    debug_n("Connection succeeded.");

    return sock;
}

int SendMessage(int socket, char* message, int message_size)
{
        if(Send(socket, &message_size, sizeof(message_size)) != 0)
        {
                close(socket);
                return 1;
        }

        if(Send(socket, message, message_size) != 0)
        {
                close(socket);
                return 1;
        }
        return 0;
}

int ReceiveMessage(int sock, char *message_buffer, int message_buffer_size)
{
    int message_size = 0;
    if(Receive(sock, &message_size, sizeof(message_size)) != 0)
    {
        debug_n("Message size receive failed.");
        free(message_buffer);
        close(sock);
    }

    if (message_size > message_buffer_size)
    {
        return -1;
    }

    // receive message
    if(Receive(sock, message_buffer, message_size) != 0)
    {
        debug_n("Receive message failed.");
        free(message_buffer);
        close(sock);
    }

    return message_size;
}

int get_sock_opt(int socket, int* tcpi_snd_cwnd, int* tcpi_retrans, int* tcpi_lost){
    struct tcp_info TCP_info;
    int tcp_info_length;
    tcp_info_length = sizeof(TCP_info);

    if ( getsockopt( socket, SOL_TCP, TCP_INFO, (void *)&TCP_info, (socklen_t *)&tcp_info_length ) == 0 )
    {
        //printf("TCPI:%u,%u,%u\n", TCP_info.tcpi_snd_cwnd, TCP_info.tcpi_retrans,TCP_info.tcpi_lost);
        *tcpi_snd_cwnd = TCP_info.tcpi_snd_cwnd;
        *tcpi_retrans =  TCP_info.tcpi_retrans;
        *tcpi_lost = TCP_info.tcpi_lost;
    }
    return 0;
}

void* tcp_stats_thread(void *data)
{
    int socket = (long) data;

    printf("THREAD STARTED");

    while(1)
    {
        int cwnd = 0;
        int retrans = 0;
        int tcpi_lost = 0;

	time_t clk = time(NULL);

        get_sock_opt(socket, &cwnd, &retrans, &tcpi_lost);

	char* timestamp = ctime(&clk);
	timestamp[strlen(timestamp) - 1] = 0;

        debug_n("tcp information: %s,%u,%u,%u.", timestamp, cwnd, retrans, tcpi_lost);

        usleep(TCP_STATS_PERIOD);
    }
}

void start_tcp_stats_collecting(int socket)
{
    pthread_t tid;
    int err = pthread_create(&tid, NULL, &tcp_stats_thread, (void*) (long) socket);
    if (err != 0)
    {
        debug_n("Can't create thread :[%s]", strerror(err));
    }

    return;
}

