#include <stdio.h>
#include <string.h>    //strlen
#include <sys/socket.h>
#include <arpa/inet.h> //inet_addr
#include <unistd.h>    //write
#include <sys/socket.h>
#include <netinet/tcp.h>
#include "common.h"

void usage();

short *read_samples(FILE* f_samples, int *count);

int parse_input_arguments(int argc, char* argv[], int *port, int *max_rate, short **samples, int *samples_count);

int  main(int argc , char *argv[])
{
    int socket_desc, client_sock , c, read_size;
    struct sockaddr_in client;
    char message[1*1024];

    // iteration duration in us
    int maxIterationDuration = 1*1000*1000;
    int *loadPerIteration = 0;

    int port = 0;
    int maxRate = 0;
    short *samples = 0;
    int samples_count = 0;
    if(parse_input_arguments(argc, argv, &port, &maxRate, &samples, &samples_count))
    {
        usage();
        return 1;
    }

    debug_n("Traffic generator started at port %d and maxRate %d bytes.", port, maxRate);

    //Create socket
    socket_desc = socket(AF_INET , SOCK_STREAM , 0);
    if (socket_desc == -1)
    {
        debug("Socket creation failed with error: %s\n", strerror(errno));
        return 1;
    }
    debug("Socket created.\n");

    if (persistent_bind(socket_desc, port))
    {
        usage();
        return 2;
    }

    //Listen
    listen(socket_desc , 3);

    //Accept and incoming connection
    debug_n("Waiting for incoming connections.");
    c = sizeof(struct sockaddr_in);

    int requestsProcessed = 0;

    //accept connection from an incoming client
    client_sock = accept(socket_desc, (struct sockaddr *)&client, (socklen_t*)&c);
    if (client_sock < 0)
    {
        debug("accept failed with error: %s\n", strerror(errno));
        return 3;
    }
    debug("Connection accepted\n");

    start_tcp_stats_collecting(client_sock);

    struct timespec referentStart;
    clock_gettime(CLOCK_MONOTONIC, &referentStart);
 

    // loop trough all samples containing percents
    for (int i = 0; ; i++)
    {
        short rate = samples[i%samples_count];

        // calculate number of iterations based on max_rate, rate and size of message. 
        int iterationsPerSecond = maxRate * (((double)rate) / 100) / sizeof(message);
        debug_n("%d: rate: %d, max_rate: %d, iterations per second: %d.", i, rate, maxRate, iterationsPerSecond);

        double uSleepTimeBetweenIterations = maxIterationDuration;
        if (iterationsPerSecond != 0)
        {
            uSleepTimeBetweenIterations = maxIterationDuration / iterationsPerSecond;
        }

        //struct timespec iterationStart;
        //clock_gettime(CLOCK_MONOTONIC, &iterationStart);
        int iterationNumber = 0;

        for(;;)
        {
            struct timespec t1, t2;

            clock_gettime(CLOCK_MONOTONIC, &t1);

            if(iterationsPerSecond > 0)
            {
                if (SendMessage(client_sock, message, sizeof(message)))
                {
                    break;
                }
            }

            clock_gettime(CLOCK_MONOTONIC, &t2);

            double iterationDuration = CalculateElapsedTime(referentStart, t2);
            iterationDuration = iterationDuration - i * maxIterationDuration;

            if (iterationDuration >= maxIterationDuration)
            {
                debug_n("Iteration count: %d, iterations per second: %d.", iterationNumber, iterationsPerSecond);
                break;
            }

            double diff = CalculateElapsedTime(t1, t2);
            double sleepTime = uSleepTimeBetweenIterations - diff;

            if(sleepTime > 0)
            {
                usleep(((int)sleepTime));
            }

            iterationNumber++;
        }
    }

    // if we are here, it means client disconnected, close socket
    debug("Client disconnected. Exiting.\n");
    return 0;
}

void usage()
{
    debug_n("Application generates load of maximum rate max_rate as defined in load_samples_file.");
    debug_n("Usage:");
    debug_n("stream_traffic_generator port max_rate load_samples_file ");
}

int parse_input_arguments(int argc, char* argv[], int *port, int *max_rate, short **samples, int *samples_count)
{
    if (argc != 4)
    {
        debug_n("Invalid number of arguments.");
        return 1;
    }

    // read input parameters
    *port = atoi(argv[1]);
    if (port <= 0)
    {
        return 2;
        debug_n("Invalid port value.");
    }

    // maxRade is provided in KB, so convert it to bytes
    *max_rate = atoi(argv[2]) * 1024 * 1;

    FILE *f_samples = fopen(argv[3], "r");
    if (f_samples == NULL)
    {
        debug_n("Failed to open samples file.");
        return 3;
    }

    *samples = read_samples(f_samples, samples_count);

    fclose(f_samples);

    return 0;
}

short *read_samples(FILE* f_samples, int *count)
{
    short *samples = NULL;

    // count the number of samples
    *count = 0;
    size_t lineLen = 32;
    char *line = (char*) malloc(lineLen);
    while (getline(&line, &lineLen, f_samples) > 0)
    {
        (*count)++;
    }

    samples = (short*) malloc(sizeof(short) * (*count));
    memset(samples, 0, sizeof(short) * (*count));

    int i = 0;
    rewind(f_samples);
    while (getline(&line, &lineLen, f_samples) > 0)
    {
        samples[i] = atoi(line);
        i++;
    }
 
    return samples;
}

